# {{ name }}

{{ description }}

## Category

{{ category }}

## Installation

```bash
uv sync
```

## Development

```bash
# Run locally
synapse run <action_name>
```

## Publishing

```bash
synapse plugin publish
```
