import type {SidebarsConfig} from '@docusaurus/plugin-content-docs';

/**
 * Synapse SDK Documentation Sidebar Configuration
 *
 * Autogenerated sidebar - 폴더 구조가 곧 사이드바 구조
 * 정렬은 _category_.json의 position과 문서의 sidebar_position으로 관리
 */
const sidebars: SidebarsConfig = {
  tutorialSidebar: [
    {
      type: 'autogenerated',
      dirName: '.',
    },
  ],
};

export default sidebars;
