[project]
name = "{{ code }}"
version = "{{ version }}"
description = "{{ description }}"
requires-python = ">=3.10"
dependencies = [
    "synapse-sdk",
]

[tool.uv]
dev-dependencies = [
    "pytest>=8.0",
    "ruff>=0.8",
]
